package com.simuwang.daq.components.report.parser.excel;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.exceptions.ExceptionUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.read.builder.ExcelReaderBuilder;
import com.simuwang.base.common.exception.ReportParseException;
import com.simuwang.base.mapper.daq.EmailFieldMappingMapper;
import com.simuwang.base.pojo.dto.report.*;
import com.simuwang.daq.components.CustomExcelMultiSheetListener;
import com.simuwang.daq.components.ReportParseUtils;
import com.simuwang.daq.components.report.parser.AbstractReportParser;

import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

public abstract class AbstractExcelReportParser<T extends ReportData> extends AbstractReportParser<T> {
    public AbstractExcelReportParser(EmailFieldMappingMapper fieldMappingMapper) {
        super(fieldMappingMapper);
    }

    @Override
    public T parse(ReportParserParams params) throws IOException, ReportParseException {
        String filename = params.getFilename();
        this.init();
        // 配置excel解析
        CustomExcelMultiSheetListener excelListener = new CustomExcelMultiSheetListener();
        try {
            ExcelReaderBuilder readerBuilder = EasyExcel.read(params.getFilepath());
            readerBuilder.sheet();
            // 没有表头
            readerBuilder.headRowNumber(0);
            // 自定义的对象，传递到事件监听器中
            readerBuilder.customObject(this.customExcelTables());
            readerBuilder.registerReadListener(excelListener);
            readerBuilder.doReadAll();
        } catch (Exception e) {
            this.logger.warn("报告解析错误：{}", ExceptionUtil.stacktraceToString(e));
            throw new ReportParseException(ReportParseStatus.NOT_A_FIXED_FORMAT, filename);
        }
        try {
            // 从excel中提取的表格信息
            List<SimpleTable> tables = excelListener.getTables();
            // 报告基本信息
            ReportBaseInfoDTO reportInfo = this.buildReportInfo(params);
            // 解析报告中主体基金的基本信息
            ReportFundInfoDTO reportFundInfo = this.buildFundInfo(params, tables);
            // 解析其他表格信息并且设置结果字段
            T reportData = this.parseExtInfoAndSetData(reportInfo, reportFundInfo, tables);
            // 数据清洗后返回
            this.cleaningReportData(reportData);
            return reportData;
        } catch (ReportParseException e) {
            throw e;
        } catch (Exception e) {
            this.logger.warn("报告解析错误：{}", ExceptionUtil.stacktraceToString(e));
            throw new ReportParseException(ReportParseStatus.NOT_A_FIXED_FORMAT, filename);
        }
    }

    @Override
    protected void cleaningReportData(T reportData) {
        // cleaning.
    }

    /**
     * 每个子类都有自己的表格解析配置
     *
     * @return /
     */
    protected abstract List<CustomExcelTable> customExcelTables();

    /**
     * 解析其他数据并设置到报告结果对象中
     *
     * @param reportInfo     报告基本信息
     * @param reportFundInfo 报告基金基本信息
     * @param tables         所有解析的表格
     * @return /
     */
    protected abstract T parseExtInfoAndSetData(ReportBaseInfoDTO reportInfo, ReportFundInfoDTO reportFundInfo, List<SimpleTable> tables);

    /**
     * 解析基金基本信息表格
     *
     * @param tables 表格
     * @return /
     */
    protected abstract ReportFundInfoDTO buildFundInfo(ReportParserParams params, List<SimpleTable> tables);

    /**
     * 构建当前基金和分级基金的数据（分级基金表格可能没有数据）
     *
     * @param fileId   文件id
     * @param tables   数据表
     * @param clazz    类对象
     * @param function 映射关系
     * @param <DTO>    /
     * @return /
     */
    protected <DTO extends BaseReportLevelDTO<?>> List<DTO> buildLevelDto(Integer fileId, List<SimpleTable> tables, Class<DTO> clazz,
                                                                          Function<SimpleTable, Map<String, Object>> function) {
        String titles = tables.stream().map(SimpleTable::getTitle).collect(Collectors.joining(","));
        List<String> levels = ReportParseUtils.matchTieredFund(titles);
        List<DTO> dtos = ListUtil.list(true);
        for (int i = 0; i < levels.size(); i++) {
            String level = levels.get(i);
            if (i >= tables.size() || StrUtil.isBlank(level)) {
                continue;
            }
            Map<String, Object> infoMap = function.apply(tables.get(i));
            DTO dto = this.buildDto(fileId, clazz, infoMap);
            if (dto == null) {
                continue;
            }
            dto.setLevel(level);
            dtos.add(dto);
        }
        return dtos;
    }
}
